<?php

namespace Civi\Themetest;

use Civi\Core\Url;
use CRM_Core_Smarty;
use CRM_Themetest_ExtensionUtil as E;
use Civi;

/**
 * Display the "Theme View Console".
 *
 * For any random page (`civicrm/foo/bar`), you may get the multi-theme console which shows
 * a comparison of `civicrm/foo/bar` across multiple themes.
 *
 * The console page is `civicrm/_tv/foo/bar`.
 *
 * Because the console is intended to be visually separate from the regular site,
 * it has its own primitive page-template.
 */
class Console {

  public static function run(): void {
    echo (new Console())->render();
    \CRM_Utils_System::civiExit();
  }

  public function render(): string {
    /** @var \Civi\Core\Url $target */
    $target = $this->createLiveUrl();

    $vars = [];
    $vars['pageTitle'] = E::ts('ThemeView Console (%1)', [1 => $target->getPath()]);
    $vars['scripts'] = [
      Civi::url('asset://[civicrm.bower]/jquery/dist/jquery.js', 'c'),
      Civi::url('asset://[civicrm.bower]/angular/angular.js', 'c'),
      Civi::url('ext://themetest/js/themeview.js', 'c'),
    ];
    $vars['styles'] = [
      Civi::url('ext://themetest/css/themeview.css', 'c'),
    ];
    $vars['tvData'] = [
      'modes' => [
        'isolated' => ['title' => 'Isolated', 'path' => $this->createIsolatedUrl($target)],
        'full' => ['title' => 'Full', 'path' => $target],
      ],
      'themes' => $this->pickThemes(),
    ];

    $pageTemplateFile = 'Civi/Themetest/Console.tpl';
    return CRM_Core_Smarty::singleton()->fetchWith($pageTemplateFile, $vars);
  }

  /**
   * Determine the URL of the ordinary page.
   *
   * We do this by checking the current request and stripping out any `_tv` flags.
   *
   * @return \Civi\Core\Url
   */
  protected function createLiveUrl(): Url {
    $config = \CRM_Core_Config::singleton();

    $target = Civi::url('backend://', 'r');
    $path = preg_replace(';civicrm/_tv/;', 'civicrm/', $config->userSystem->currentPath());
    $target->setPath($path);
    $route = \CRM_Core_Menu::get($path);
    if (!empty($route['is_public'])) {
      $target->setScheme('frontend');
    }

    foreach ($_GET as $key => $value) {
      if ($key[0] !== '_' && $key !== $config->userFrameworkURLVar) {
        $target->addQuery([$key => $value]);
      }
    }

    return $target;
  }

  /**
   * Take a regular URL and make a variant which hides regular page-chrome from CMS.
   *
   * @param \Civi\Core\Url $target
   *   Ex: 'civicrm/foo/bar'
   * @return \Civi\Core\Url|null
   *   Ex: 'civicrm/foo/bar?_tv=isolated'
   */
  protected function createIsolatedUrl(Url $target): ?Url {
    return (clone $target)->addQuery(['_tv' => 'isolated']);
  }

  /**
   * Get a list of available themes.
   *
   * @return array{array{name: string, title: string, active: bool}}
   */
  protected function pickThemes(): array {
    /** @var \Civi\Core\Themes $themes */
    $themes = Civi::service('themes');
    $result = [];
    foreach ($themes->getAll() as $name => $theme) {
      if ($name[0] !== '_' && $name !== 'default') {
        $result[] = [
          'name' => $theme['name'],
          'title' => $theme['title'],

          // Active flag only matters on first page-view. Thereafter, localStorage takes precedence.
          'active' => $theme['name'] === $themes->getActiveThemeKey(),
        ];
      }
    }
    usort($result, function ($a, $b) {
      return strnatcasecmp($a['name'], $b['name']);
    });
    return $result;
  }

}
