(function (angular, $) {
  const tv = angular.module('themeview', []);

  function getTpl(selector) {
    return () => $(selector).html();
  }

  /**
   * Whenever the primary frame scrolls, we should update the position of the other frames.
   *
   * @param source
   *   The iframe which just moved
   * @param iframes
   *   Array of all iframe which should be synchronized.
   */
  function syncScroll(source, iframes) {
    const sourceDoc = source.contentDocument || source.contentWindow.document;
    if (!sourceDoc) return;

    const sourceScrollTop = sourceDoc.documentElement.scrollTop || sourceDoc.body.scrollTop;
    const sourceScrollHeight = sourceDoc.documentElement.scrollHeight - sourceDoc.documentElement.clientHeight;

    const sourceScrollLeft = sourceDoc.documentElement.scrollLeft || sourceDoc.body.scrollLeft;
    const sourceScrollWidth = sourceDoc.documentElement.scrollWidth - sourceDoc.documentElement.clientWidth;

    const verticalScrollRatio = sourceScrollTop / sourceScrollHeight;
    const horizontalScrollRatio = sourceScrollLeft / sourceScrollWidth;

    iframes.forEach(target => {
      if (target === source) return; // Skip syncing the source iframe

      const targetDoc = target.contentDocument || target.contentWindow.document;
      if (!targetDoc) return;

      const targetScrollHeight = targetDoc.documentElement.scrollHeight - targetDoc.documentElement.clientHeight;
      const targetScrollWidth = targetDoc.documentElement.scrollWidth - targetDoc.documentElement.clientWidth;

      targetDoc.documentElement.scrollTop = verticalScrollRatio * targetScrollHeight;
      targetDoc.body.scrollTop = verticalScrollRatio * targetScrollHeight; // For compatibility

      targetDoc.documentElement.scrollLeft = horizontalScrollRatio * targetScrollWidth;
      targetDoc.body.scrollLeft = horizontalScrollRatio * targetScrollWidth; // For compatibility
    });
  }

  function buildThemes() {
    const themes = angular.copy(window.tvData.themes);
    if (localStorage.getItem('tv-iframe-themes')) {
      const activeThemes = localStorage.getItem('tv-iframe-themes').split(/,/);
      themes.forEach((t) => t.active = activeThemes.includes(t.name));
    }
    return themes;
  }

  tv.component('tvMain', {
    // bindings: {},
    template: getTpl('#tv-main'),
    controller: function ($scope, $element, $location) {
      const initialStyle = getComputedStyle(document.documentElement);
      const liveStyle = document.documentElement.style;
      const defaultMode = Object.keys(window.tvData.modes)[0];
      $scope.modes = window.tvData.modes;
      $scope.themes = buildThemes();

      $scope.iframeSpec = {
        mode: localStorage.getItem('tv-iframe-mode') || defaultMode,
        width: localStorage.getItem('tv-iframe-width') || initialStyle.getPropertyValue('--iframe-width'),
        height: localStorage.getItem('tv-iframe-height') || initialStyle.getPropertyValue('--iframe-height')
      };
      liveStyle.setProperty('--iframe-width', $scope.iframeSpec.width);
      liveStyle.setProperty('--iframe-height', $scope.iframeSpec.height);

      $scope.$watch('iframeSpec.mode', (newVal, oldVal) => {
        localStorage.setItem('tv-iframe-mode', newVal);
        if (newVal != oldVal) $scope.redrawIframes();
      });
      $scope.$watch('iframeSpec.width', (newVal, oldVal) => {
        localStorage.setItem('tv-iframe-width', newVal);
        if (newVal != oldVal) liveStyle.setProperty('--iframe-width', newVal);
      });
      $scope.$watch('iframeSpec.height', (newVal, oldVal) => {
        localStorage.setItem('tv-iframe-height', newVal);
        if (newVal != oldVal) liveStyle.setProperty('--iframe-height', newVal);
      });

      $scope.onToggleTheme = (theme) => {
        localStorage.setItem('tv-iframe-themes', $scope.themes.filter((t) => t.active).map((t) => t.name).join(','));
        $scope.redrawIframes();
      };

      $scope.getUrl = (themeName) => {
        const basePath = $scope.modes[$scope.iframeSpec.mode].path;
        const sep = basePath.match(/\?/) ? '&' : '?';
        var result = basePath + sep + '_tvTheme=' + encodeURIComponent(themeName);
        if ($location.hash()) {
          result += '#' + $location.hash();
        }
        return result;
      };

      $scope.redrawIframes = () => {
        const iframes = [];
        $scope.themes.forEach(theme => {
          if (!theme.active) return;
          const iframe = $('<iframe>')[0];
          iframe.src = $scope.getUrl(theme.name);
          // For the first frame, we monitor scrolling - and sync to other frames.
          if (iframes.length < 1) {
            iframe.addEventListener('load', () => {
              iframe.contentWindow.addEventListener('scroll', () => syncScroll(iframe, iframes));
            });
          }
          iframes.push(iframe);
        });

        const container = $element.find('.container');
        container.children().remove();
        container.append(iframes);
      };

      $scope.redrawIframes();
    }

  });

})(angular, jQuery);
